/*
 * GStreamer
 * Copyright (C) 2015 Dimitrios Katsaros <patcherwork@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Alternatively, the contents of this file may be used under the
 * GNU Lesser General Public License Version 2.1 (the "LGPL"), in
 * which case the following provisions apply instead of the ones
 * mentioned above:
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */



#ifndef __GST_HIST_META_H__
#define __GST_HIST_META_H__

#include <gst/gst.h>

#define GST_HIST_META_IMPL_NAME "HistMeta"

typedef struct _GstHistMeta GstHistMeta;

#define MAX_NR_BINS 65536

struct _GstHistMeta {
    GstMeta        meta;
    guint          *bins;
    guint          bin_no;
    guint16        minval; //min img intensity
    guint16        maxval; //max img intensity
    gdouble        avgval;
    gdouble        stddevval; //normalized stddev = stddev/avg
    gint           medianid;
    gint           modeid; //bin with most pixels
    guint16        satval;
    gulong	   total_pixels;
};


GType gst_hist_meta_api_get_type (void);
#define GST_HIST_META_API_TYPE (gst_hist_meta_api_get_type())

#define gst_buffer_get_gst_hist_meta(b) \
	((GstHistMeta*)gst_buffer_get_meta((b),GST_HIST_META_API_TYPE))

const GstMetaInfo *gst_hist_meta_get_info (void);
#define GST_HIST_META_INFO (gst_hist_meta_get_info())

#define BIN_WIDTH(meta)         ((guint)(meta->satval + 1) / meta->bin_no)
#define BIN_START(meta, index)  ((guint)BIN_WIDTH(meta)*index)
#define BIN_END(meta, index)    ((guint)BIN_WIDTH(meta)*(index + 1) - 1)
#define BIN_MID(meta, index)    (BIN_WIDTH(meta)*(index + 0.5) - 0.5)
#define BIN_PERCENT(meta, index) ((100.0*meta->bins[index])/meta->total_pixels)

GstHistMeta * gst_buffer_add_gst_hist_meta (GstBuffer      *buffer,
                                            guint          *bins,
                                            guint           bin_no,
                                            guint16         minval,
                                            guint16         maxval,
                                            gdouble         avgval,
					                        gdouble         stddevval,
                                            gint            medianid,
                                            gint            modeid,
                                            guint16         satval,
					                        gulong	    total_pixels);

gchar * gst_hist_meta_to_string(GstHistMeta *meta);
gchar * gst_hist_meta_bin_to_string(GstHistMeta *meta, guint index);

#endif /* __GST_HIST_META_H__ */
